@@ -80,8 +80,9 @@ soup_message_flags_get_type (void)
 	static GType etype = 0;
 	if (G_UNLIKELY (etype == 0)) {
 		static const GFlagsValue values[] = {
-			{ SOUP_MESSAGE_OVERWRITE_CHUNKS, "SOUP_MESSAGE_OVERWRITE_CHUNKS", "overwrite-chunks" },
 			{ SOUP_MESSAGE_NO_REDIRECT, "SOUP_MESSAGE_NO_REDIRECT", "no-redirect" },
+			{ SOUP_MESSAGE_OVERWRITE_CHUNKS, "SOUP_MESSAGE_OVERWRITE_CHUNKS", "overwrite-chunks" },
+			{ SOUP_MESSAGE_CONTENT_DECODED, "SOUP_MESSAGE_CONTENT_DECODED", "content-decoded" },
 			{ 0, NULL, NULL }
 		};
 		etype = g_flags_register_static (g_intern_static_string ("SoupMessageFlags"), values);
@@ -116,6 +117,7 @@ soup_message_headers_type_get_type (void)
 		static const GEnumValue values[] = {
 			{ SOUP_MESSAGE_HEADERS_REQUEST, "SOUP_MESSAGE_HEADERS_REQUEST", "request" },
 			{ SOUP_MESSAGE_HEADERS_RESPONSE, "SOUP_MESSAGE_HEADERS_RESPONSE", "response" },
+			{ SOUP_MESSAGE_HEADERS_MULTIPART, "SOUP_MESSAGE_HEADERS_MULTIPART", "multipart" },
 			{ 0, NULL, NULL }
 		};
 		etype = g_enum_register_static (g_intern_static_string ("SoupMessageHeadersType"), values);
