@@ -64,6 +64,8 @@ typedef struct {
 
 	GSList      *connections;      /* CONTAINS: SoupConnection */
 	guint        num_conns;
+
+	guint        num_messages;
 } SoupSessionHost;
 
 typedef struct {
@@ -880,10 +882,12 @@ redirect_handler (SoupMessage *msg, gpointer user_data)
 	     !SOUP_METHOD_IS_SAFE (msg->method)) ||
 	    (msg->status_code == SOUP_STATUS_MOVED_PERMANENTLY &&
 	     msg->method == SOUP_METHOD_POST)) {
-		/* Redirect using a GET */
-		g_object_set (msg,
-			      SOUP_MESSAGE_METHOD, SOUP_METHOD_GET,
-			      NULL);
+		if (msg->method != SOUP_METHOD_HEAD) {
+			/* Redirect using a GET */
+			g_object_set (msg,
+				      SOUP_MESSAGE_METHOD, SOUP_METHOD_GET,
+				      NULL);
+		}
 		soup_message_set_request (msg, NULL,
 					  SOUP_MEMORY_STATIC, NULL, 0);
 		soup_message_headers_set_encoding (msg->request_headers,
@@ -1044,8 +1048,10 @@ soup_session_connection_failed (SoupSession *session,
 }
 
 static void
-tunnel_connected (SoupSession *session, SoupMessage *msg, gpointer user_data)
+tunnel_connected (SoupMessage *msg, gpointer user_data)
 {
+	SoupSession *session = user_data;
+
 	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
 		SoupSessionPrivate *priv = SOUP_SESSION_GET_PRIVATE (session);
 		SoupMessageQueueItem *item =
@@ -1057,6 +1063,7 @@ tunnel_connected (SoupSession *session, SoupMessage *msg, gpointer user_data)
 		g_object_set (item->conn,
 			      SOUP_CONNECTION_PROXY_URI, NULL,
 			      NULL);
+		soup_message_queue_item_unref (item);
 	}
 }
 
@@ -1080,9 +1087,13 @@ soup_session_make_connect_message (SoupSession *session,
 
 	/* Call the base implementation of soup_session_queue_message
 	 * directly, to add msg to the SoupMessageQueue and cause all
-	 * the right signals to be emitted.
+	 * the right signals to be emitted. We can't use
+	 * queue_message's callback arg in this case because that's
+	 * actually implemented by the subclasses.
 	 */
-	queue_message (session, msg, tunnel_connected, NULL);
+	g_signal_connect (msg, "finished",
+			  G_CALLBACK (tunnel_connected), session);
+	queue_message (session, msg, NULL, NULL);
 	item = soup_message_queue_lookup (priv->queue, msg);
 	g_object_unref (msg);
 	return item;
@@ -1134,7 +1145,7 @@ soup_session_get_connection (SoupSession *session,
 	SoupAddress *remote_addr, *tunnel_addr;
 	SoupSSLCredentials *ssl_creds;
 	GSList *conns;
-	gboolean has_pending = FALSE;
+	int num_pending = 0;
 	SoupURI *uri;
 
 	g_mutex_lock (priv->host_lock);
@@ -1146,14 +1157,13 @@ soup_session_get_connection (SoupSession *session,
 			g_mutex_unlock (priv->host_lock);
 			return conns->data;
 		} else if (soup_connection_get_state (conns->data) == SOUP_CONNECTION_CONNECTING)
-			has_pending = TRUE;
+			num_pending++;
 	}
 
-	if (has_pending) {
-		/* We've already started one connection to this
-		 * address, so don't start another one until it's
-		 * done.
-		 */
+	/* Limit the number of pending connections; num_messages / 2
+	 * is somewhat arbitrary...
+	 */
+	if (num_pending > host->num_messages / 2) {
 		g_mutex_unlock (priv->host_lock);
 		return NULL;
 	}
@@ -1225,6 +1235,7 @@ message_finished (SoupMessage *msg, gpointer user_data)
 	SoupMessageQueueItem *item = user_data;
 	SoupSession *session = item->session;
 	SoupSessionPrivate *priv = SOUP_SESSION_GET_PRIVATE (session);
+	SoupSessionHost *host;
 
 	if (item->conn) {
 		g_object_unref (item->conn);
@@ -1233,6 +1244,12 @@ message_finished (SoupMessage *msg, gpointer user_data)
 
 	if (!SOUP_MESSAGE_IS_STARTING (msg)) {
 		soup_message_queue_remove (priv->queue, item);
+
+		g_mutex_lock (priv->host_lock);
+		host = get_host_for_message (session, item->msg);
+		host->num_messages--;
+		g_mutex_unlock (priv->host_lock);
+
 		g_signal_handlers_disconnect_by_func (msg, message_finished, item);
 		/* g_signal_handlers_disconnect_by_func doesn't work if you
 		 * have a metamarshal, meaning it doesn't work with
@@ -1251,8 +1268,15 @@ queue_message (SoupSession *session, SoupMessage *msg,
 {
 	SoupSessionPrivate *priv = SOUP_SESSION_GET_PRIVATE (session);
 	SoupMessageQueueItem *item;
+	SoupSessionHost *host;
 
 	item = soup_message_queue_append (priv->queue, msg, callback, user_data);
+
+	g_mutex_lock (priv->host_lock);
+	host = get_host_for_message (session, item->msg);
+	host->num_messages++;
+	g_mutex_unlock (priv->host_lock);
+
 	soup_message_set_io_status (msg, SOUP_MESSAGE_IO_STATUS_QUEUED);
 
 	g_signal_connect_after (msg, "finished",
